<?php

/**
 * Albireo Kernel
 *
 * @copyright  Copyright (c) 2010 Albireo Solutions
 * @package    Kernel
 */

namespace ApplicationsModule\BooksModule;

use Nette\Object;
use \dibi;
use \InvalidArgumentException;
use Nette\Collections\ArrayList;
use Nette\Environment;

class BookManager extends Object {

    /**
     * returns books with authors
     * @return type 
     */
    public function getBooks() {
        $res = $this->getBookDataSource()->fetchAssoc('id_book', 'b.id_author');
        return $this->buildBooks($res);
    }

    public function getBooksByAuthor($id, $lang = 'cs') {
        $res = $this->getBookDataSource()->where("a.id_author = %i", $id)->and("language = %s", $lang)->fetchAll();
        return $this->buildBooks($res);
    }

    public function getBookDataSource() {
        return dibi::select('b.id_book as id_book,
                    url,
                    title,
                    a.id_author as id_author,
                    firstname,
                    surname,
                    language')->from('[books] b
                LEFT JOIN [books_authors] ba ON (b.id_book = ba.id_book)
                LEFT JOIN [authors] a ON (a.id_author = ba.id_author)');
    }

    public function lastRead($userId) {
        $res = $this->getBookDataSource()->join("[books_last_read] blr ON (b.id_book = blr.id_book) WHERE blr.id_user = $userId")->orderBy("time", dibi::DESC)->fetchAssoc('id_book', 'b.id_author');
        
        return $this->buildBooks($res);
    }
    
    public function addLast(Book $book, $userId) {
        $row = dibi::select("*")->from("books_last_read")->where("id_user = $userId AND id_book = ".$book->id)->fetch();
        if ($row) {
            dibi::query("UPDATE books_last_read SET [time] = CURRENT_TIMESTAMP where id_user = $userId AND id_book = ".$book->id);
        } else {
            dibi::query("INSERT INTO books_last_read(id_book, id_user) VALUES (".$book->id.", $userId)");
        }
    }

    /**
     *  returns only authors (without books for now)
     * 
     * @return type 
     */
    public function getAuthors($lang = 'cs') {
        $res = dibi::query("SELECT a.id_author, firstname, surname, count(*) AS bookcount FROM [authors] a JOIN [books_authors] ba ON a.id_author = ba.id_author JOIN [books] b ON b.id_book = ba.id_book WHERE language='$lang' GROUP BY surname,firstname,a.id_author ORDER BY surname")->fetchAll();
        return $this->buildAuthors($res);
    }

    public function getBook($id) {
        return $this->buildBook(dibi::query("SELECT b.*, ba.id_author FROM [books] b JOIN [books_authors] ba ON [b.id_book] = [ba.id_book] WHERE b.id_book = %i", $id)->fetch());
    }

    /**
     * @param ApplicationsModule\BooksModule\Book $book
     */
    public function addBook(Book $book) {
        if (is_null($book->getUrl()) || is_null($book->getTitle()) || is_null($book->getAuthors())) {
            throw new InvalidArgumentException(_("Book object must contain title, url to book dir and author"));
        }

        dibi::begin();
        dibi::query("INSERT INTO [books]", array(
            "title" => $book->getTitle(),
            "url" => $book->getUrl()
        ));
        $book->setId(dibi::getInsertId());
        foreach ($book->getAuthors() as $author) {
            $this->addAuthor($author);
            $this->addAuthorToBook($author, $book);
        }
        dibi::commit();
    }

    public function addAuthor(Author $author) {
        $res = dibi::fetch("SELECT * FROM [authors] WHERE " . ( is_null($author->getFirstname()) ? "" : "[firstname]='" . $author->getFirstname() . "' AND " ) . "[surname]=%s", $author->getSurname());
        if ($res) {
            $author->setId((int) $res["id_author"]);
            return true;
        } else {
            dibi::query("INSERT INTO [authors]([firstname], [surname])
                    VALUES (%s,%s)", $author->getFirstname(), $author->getSurname());
            $author->setId(dibi::getInsertId());
            return true;
        }
    }

    public function addAuthorToBook(Author $author, Book $book) {
        dibi::query("INSERT INTO [books_authors] (id_author, id_book) VALUES(%i,%i)", $author->getId(), $book->getId());
    }

    public function addOneAuthorBook($first, $sur, $title, $url) {
        $book = new Book();
        $book->setTitle($title);
        $book->setUrl($url);
        $author = new Author();
        $author->setFirstname($first);
        $author->setSurname($sur);
        $authors = new ArrayList();
        $authors->append($author);
        $book->setAuthors($authors);

        $this->addBook($book);
    }

    /**
     * @param ApplicationsModule\BooksModule\Book $book
     * @return void
     */
    public function removeBook(Book $book) {
        return dibi::query("DELETE FROM [books] WHERE [id_book] = %i OR [url] = %s", $book->getId(), $book->getUrl());
    }

    /**
     * @param ApplicationsModule\BooksModule\Author $author
     * @return void
     */
    public function removeAuthor(Author $author) {
        return dibi::query("DELETE FROM [authors] WHERE [id_author] = %i", $author->getId());
    }

    private function buildBooks($res) {
        $books = new ArrayList();
        foreach ($res as $n => $row) {
            $book = $this->buildBook($row);
            $books->append($book);
        }
        return $books;
    }

    private function buildBook($row) {
        $book = new Book();
        $book->setId($row["id_book"]);
        $book->setTitle($row["title"]);
        $book->setUrl($row["url"]);
        //TODO: temporary -> only one author!!!
        if (isset($row["id_author"])) {
            $author = new Author();
            $author->setId($row["id_author"]);
            if (isset($row["firstname"])) {
                $author->setFirstname($row["firstname"]);
            }
            if (isset($row["firstname"])) {
                $author->setSurname($row["surname"]);
            }
            $book->setAuthors(array($author));
        }
        return $book;
    }

    private function buildAuthors($res) {
        $authors = new ArrayList();
        foreach ($res as $n => $row) {
            $author = $this->buildAuthor($row);
            $authors->append($author);
        }
        return $authors;
    }

    private function buildAuthor($row) {
        $author = new Author();
        $author->setId($row["id_author"]);
        $author->setFirstname($row["firstname"]);
        $author->setSurname($row["surname"]);
        if ($row["bookcount"]) {
            $author->setBookCount($row["bookcount"]);
        }
        return $author;
    }

}